@@ -264,7 +264,7 @@ gail_finish_select (GtkWidget *widget)
    * We may already have got a notification because of bug 68483
    */
     return;
-  
+
   /*
    * If previously focused widget is not a GtkMenuItem or a GtkMenu,
    * keep track of it so we can return to it after menubar is deactivated
@@ -311,7 +311,7 @@ gail_deselect_watcher (GSignalInvocationHint *ihint,
       GtkWidget *parent_menu_shell;
 
       parent_menu_shell = GTK_MENU_SHELL (menu_shell)->parent_menu_shell;
-      if (GTK_IS_MENU (parent_menu_shell))
+      if (parent_menu_shell)
         {
           GtkWidget *active_menu_item;
 
@@ -321,6 +321,10 @@ gail_deselect_watcher (GSignalInvocationHint *ihint,
               gail_focus_notify_when_idle (active_menu_item);
             }
         }
+      else
+        {
+          gail_focus_notify_when_idle (menu_shell);
+        }
     }
   return TRUE; 
 }
@@ -379,7 +383,15 @@ gail_focus_notify_when_idle (GtkWidget *widget)
 {
   if (focus_notify_handler)
     {
-      gtk_idle_remove (focus_notify_handler);
+      if (widget)
+        {
+          gtk_idle_remove (focus_notify_handler);
+        }
+      else
+        /*
+         * Ignore if focus is being set to NULL and we are waiting to set focus
+         */
+        return;
     }
 
   focus_notify_handler = gtk_idle_add (gail_focus_idle_handler, widget);
