@@ -109,15 +109,20 @@ gail_text_util_text_setup (GailTextUtil *textutil,
 {
   g_return_if_fail (GAIL_IS_TEXT_UTIL (textutil));
 
-  if (textutil->buffer != NULL)
+  if (textutil->buffer)
     {
-      g_object_unref (textutil->buffer);
-      textutil->buffer = NULL;
+      if (!text)
+        {
+          g_object_unref (textutil->buffer);
+          textutil->buffer = NULL;
+          return;
+        }
+    }
+  else
+    {
+      textutil->buffer = gtk_text_buffer_new (NULL);
     }
-  if (text == NULL)
-    return;
 
-  textutil->buffer = gtk_text_buffer_new (NULL);
   gtk_text_buffer_set_text (textutil->buffer, text, -1);
 }
 
@@ -163,9 +168,8 @@ gail_text_util_get_text (GailTextUtil    *textutil,
                          gint            *start_offset,
                          gint            *end_offset)
 {
-  GtkTextIter start, end, empty_check;
+  GtkTextIter start, end;
   gint line_number;
-  gboolean not_empty;
   GtkTextBuffer *buffer;
 
   g_return_val_if_fail (GAIL_IS_TEXT_UTIL (textutil), NULL);
@@ -178,17 +182,14 @@ gail_text_util_get_text (GailTextUtil    *textutil,
       return NULL;
     }
 
-  gtk_text_buffer_get_iter_at_offset (buffer, &start, offset);
-
-  gtk_text_buffer_get_iter_at_offset (buffer, &empty_check, 0);
-  not_empty = gtk_text_iter_forward_char (&empty_check);
-
-  if (!not_empty)
+  if (!gtk_text_buffer_get_char_count (buffer))
     {
       *start_offset = 0;
       *end_offset = 0;
       return g_strdup ("");
     }
+  gtk_text_buffer_get_iter_at_offset (buffer, &start, offset);
+
     
   end = start;
 
@@ -407,13 +408,15 @@ gail_text_util_get_text (GailTextUtil    *textutil,
             }
           else if GTK_IS_TEXT_VIEW (layout)
             {
-              /*
-               * It is assumed that bug 67356 is fixed.
-               */
               GtkTextView *view = GTK_TEXT_VIEW (layout);
 
               gtk_text_view_backward_display_line_start (view, &start);
-              gtk_text_view_forward_display_line (view, &end);
+              /*
+               * The call to gtk_text_iter_forward_to_end() is needed
+               * because of bug 81960
+               */
+              if (!gtk_text_view_forward_display_line (view, &end))
+                gtk_text_iter_forward_to_end (&end);
             }
           else if PANGO_IS_LAYOUT (layout)
             get_pango_text_offsets (PANGO_LAYOUT (layout),
